@@ -11,9 +11,6 @@
 # MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 #
-sigmoid_rcpp <- function (x)
-  .Call('varbvs_sigmoid_rcpp',x)
-
 varbvsnormupdate_rcpp <- function(X, sigma, sa, logodds, xy, d,
                                   alpha, mu, Xr, i)
   invisible(.Call(C_varbvs_varbvsnormupdate_rcpp,X,sigma,sa,
